## packages from Lindsey's www.commanster.eu/rcode.html
library("gnlm")
library("repeated")
## "gnlmix4MMM.R" is a modified gnlmix() function.  The following are the edits to
## Lindsey's original gnlmix() in order to get a function
## that worked for MMMs where the conditional model is a random intercept model:
## To make gnlmix4MMM.R, edit gnlmix.R of repeated package in two places:
##  np <- npl + nps + 1         becomes   np <- npl
##   p <- c(pmu, pshape, pmix)  becomes    p <- pmu
## These are the only edits required to estimate a MMM, which puts the variance
## component into the non-linear predictor of the mean.  Warnings/errors still might
## be generated about Mixing Dispersion Parameters and Correlations because of these
## modifications.  These are superficial:  The Mixing Dispersion parameter is now
## listed in the location parameters and the correlations can be obtained by
## directly accessing fitted.obj$corr; the error relates to assigning dimnames.  See examples below.
## One warning message persists for LLB, which can be disregarded:
## Warning message:
##In nlm(like, p = p, hessian = TRUE, print.level = print.level, typsize = typsize,  :
##  NA/Inf replaced by maximum positive value
source("gnlmix4MMM.R")


## gnlmix uses romberg integration and can take some time to run but is essentially exact,
## as noted by Jim Lindsey:
##https://stat.ethz.ch/pipermail/r-help/2001-May/013037.html
## The LPN example below takes 30 minutes to run.
## The LLB example below takes 35 minutes to run.
## Time can be lowered by using fewer points; points=6 may be overkill for this example.
## Lowering to points=2 reduces time only by 5 minutes, estimates are same to 3 decimal places.

## read in baltimore visual impairment data
d <- read.csv("bwVI.csv", T)
## have to attach for gnlmix() to work
attach(d)


## need outcomes to be in two column format
## # diseased in first
## # healthy  in second
y <- cbind(d$value,(1-d$value))


## Caffo and Griswold (2006) marginal link-probit-normal model, with a marginal logit-link.
## Variance pmix is specified on log scale; random intercept rand is an entity to be
## specified in the mean mu.
## mu is complicated, but working from the outside in, left to right:
## pnorm() is the conditional inverse link (aka conditional CDF)
## The resulting integral of marginalizing a conditional probit model with normally distributed
## random intercept is a normal distribution with additive means and additive variances.
## Therefore the Delta function can be written as the inverse normal with variance 1+tau^2 applied
## to the marginal inverse link (aka marginal distribution, logistic: 1/(1+exp())).
## Due to Normal distributions being closed under scaling, we can equivalently write
## Delta as a standard Normal inverse (qnorm()) and scale the operators, which we do with tau^2 = exp(pmix).
LPN  <- gnlmix4MMM(   y = y,
           distribution = "binomial",
                mixture = "normal",
                 random = "rand",
                   nest = id,
                     mu = ~pnorm(qnorm(1/(1+exp(-a0 - a1*black)))*sqrt(1+exp(pmix)) + rand),
                    pmu = c(-2.0, 0.6, log(3^2)),
                   pmix = log(3^2),
                gradtol = 1e-5,
                steptol = 1e-5,
                iterlim = 1e5,
                    eps = 1e-4,
                 points = 6)

print("call")
LPN$call
print("likelihood")
LPN$maxlike
print("iterations")
LPN$iterations
print("code: 1 -best 2-ok 3,4,5 - problem")
LPN$code
print("coefficients")
LPN$coeff
print("se")
LPN$se
print("corr")
LPN$corr
## print everything, even including some slots that are empty due to
## edits performed on gnlmix() to render gnlmix4MMM()
print(LPN)




## Wang and Louis (2003) logit-logit-bridge model...in R!  Their paper provided SAS code.
## the "mu" specification is especially busy, but a work around was not easily found.
## A careful look shows structure:  mu = 1/(1+exp(-Delta - random_intercept)
## 1/(1+exp(-eta)) is the conditional inverse logit-link (logistic CDF)
## eta = Delta + random_intercept
## where Delta can be simplified to a scaling of marginal parameters a0 and a1
## because marginalizing the conditional logistic model with Bridge-distributed random intercept
## yields a logistic distribution with inflated variance; since logistic distribution is closed under
## scaling, it can be scaled back to the standard logistic to yield the marginal link.  
## which allows conditional specification:              a0_c + a1_c*black
## to be represented in terms of marginal coefficients:(a0   + a1  *black)*sqrt(1+3/pi/pi*exp(pmix))
## and a0, a1 are the marginal parameters.  pmix is the variance on the log scale.
## where random_intercept is the probabilty integral transform of rand ~ N(0, exp(pmix))
## to give an entity with the Bridge distribution.  Using pnorm(rand/sqrt(exp(pmix))) gives
## a Uniform(0,1) that is non-linear tranformed by the inverse CDF of the Bridge distibution.

LLB  <- gnlmix4MMM(   y = y,
           distribution = "binomial",
                mixture = "normal",
                 random = "rand",
                   nest = id,
                     mu = ~ 1/(1+exp(-(a0 + a1*black)*sqrt(1+3/pi/pi*exp(pmix)) - sqrt(1+3/pi/pi*exp(pmix))*log(sin(pi*pnorm(rand/sqrt(exp(pmix)))/sqrt(1+3/pi/pi*exp(pmix)))/sin(pi*(1-pnorm(rand/sqrt(exp(pmix))))/sqrt(1+3/pi/pi*exp(pmix)))))),
                    pmu = c(-2.0, 0.6, log(3^2)),
                   pmix = log(3^2),
                gradtol = 1e-5,
                steptol = 1e-5,
                iterlim = 1e5,
                    eps = 1e-4,
                 points = 6)

print("call")
LLB$call
print("likelihood")
LLB$maxlike
print("iterations")
LLB$iterations
print("code: 1 -best 2-ok 3,4,5 - problem")
LLB$code
print("coefficients")
LLB$coeff
print("se")
LLB$se
print("corr")
LLB$corr
## print everything, even including some slots that are empty due to
## edits performed on gnlmix() to render gnlmix4MMM()
print(LLB)


## Heagerty (1999) marginally specifies a logit link and has a nonlinear conditional model
## that lacks a straightforward, linear interpretation.  A package, lnMLE is available
## for UNIX based systems at: http://faculty.washington.edu/heagerty/Software/LDA/
## the following code will not run if lnMLE is not successfully installed.
library(lnMLE)
m <- read.csv("bwVI.csv", T)
m.df <- as.data.frame(m)
model1 <- logit.normal.mle(meanmodel = value ~ black,
                           logSigma= ~1,
                           id=m$id,
                           model="marginal",
                           data=m.df,
                           tol=1e-5,
                           maxits=100,
                           r=50)
print.logit.normal.mle(model1)

